#include <windows.h>
#include <stdlib.h>
#include <stdio.h>

/* Prototypes */
void usage (void);
int recurse_files (char *dir);
BOOL set_file_time (char *pathname, char *filename, FILETIME *filetime);
BOOL find_file_time (char *dir, char *filename, FILETIME *filetime);

/* Globals */
BOOL subdirs = FALSE;
char *sourcedir = NULL;

int main (int argc, char *argv[]) {

	int i;
   char *dir = NULL, **p;

   /* Lue komentorivi */
   for (p = &argv[1]; *p != NULL; p++) {
   	if ((*p)[0] == '/') {
      	if (toupper((*p)[1]) == 'S' && (*p)[2] == '\0') subdirs = TRUE;
         else usage ();
      } else if (dir == NULL) dir = *p;
      else if (sourcedir == NULL) sourcedir = *p;
      else usage ();
   }
   if (dir == NULL) usage ();

   i = recurse_files (dir);
   printf ("%u file time(s) successfully reset.\n", i);
   return EXIT_SUCCESS;
}

void usage (void) {
   printf (
   	"Resets file creation and modification time stamps.\n\n"
   	"FILETIME [drive:]path [[drive:]originalpath] [/S]\n\n"
      "  [drive:]path\n"
      "              Directory of files to modify.\n"
      "  [drive:]originalpath\n"
      "              Directory containing files with original creation time stamps\n"
      "  /S          Modifies files in specified directory and all subdirectories\n\n"
      "If original file directory is not specified, the modification time of the\n"
      "file(s) will be reset to the creation time. Otherwise a file with same name\n"
      "is searched in the original file directory (and if /S is specified, in all\n"
      "subdirectories), and the target file creation and modification time is reset\n"
      "to the creation date of that original file.\n");
	exit (EXIT_FAILURE);
}

/* Ky lpi kaikki tiedostot pyydetyss hakemistossa */
int recurse_files (char *dir) {

	WIN32_FIND_DATA wfd;
   HANDLE h;
	char pathname[MAX_PATH], *ptr;
   int files = 0;

   strcpy (pathname, dir);
	strcat (pathname, "\\");
   ptr = strchr (pathname, '\0');

   strcpy (ptr, "*.*");
   h = FindFirstFile (pathname, &wfd);
   if (h == INVALID_HANDLE_VALUE) return 0;

   do {
      strcpy (ptr, wfd.cFileName);
   	if (wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
      	if (subdirs && strcmp (wfd.cFileName, ".") != 0 &&
         	strcmp (wfd.cFileName, "..") != 0) {

      		files += recurse_files (pathname);
         }
      } else {
      	if (set_file_time (pathname, wfd.cFileName, &wfd.ftCreationTime)) {
         	files++;
         }
      }
   } while (FindNextFile (h, &wfd));
   FindClose (h);
   return files;
}

/* Aseta pyydetyn tiedoston aikaleima */
BOOL set_file_time (char *pathname, char *filename, FILETIME *filetime) {

	HANDLE h;

   if (sourcedir != NULL) {
   	if (!find_file_time (sourcedir, filename, filetime)) {
      	fprintf (stderr, "Cannot find source file for '%s'.\n", pathname);
         return FALSE;
      }
   }
	h = CreateFile (pathname, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (h == INVALID_HANDLE_VALUE) {
   	fprintf (stderr, "Cannot reset file time of '%s'.\n", pathname);
      return FALSE;
   }
   /* Aseta tiedoston CreationTime ja LastWriteTime pyydettyyn arvoon */
   SetFileTime (h, filetime, NULL, filetime);
   CloseHandle (h);
   return TRUE;
}

/* Hae tietyn nimisen tiedoston luontiaika */
BOOL find_file_time (char *dir, char *filename, FILETIME *filetime) {

	WIN32_FIND_DATA wfd;
   HANDLE h;
	char pathname[MAX_PATH], *ptr;

   strcpy (pathname, dir);
	strcat (pathname, "\\");
   ptr = strchr (pathname, '\0');

   strcat (pathname, filename);
   h = FindFirstFile (pathname, &wfd);
   if (h != INVALID_HANDLE_VALUE) {
   	*filetime = wfd.ftCreationTime;
      FindClose (h);
      return TRUE;
   }

   strcpy (ptr, "*.*");
   h = FindFirstFile (pathname, &wfd);
   if (h == INVALID_HANDLE_VALUE) return FALSE;

   do {
      strcpy (ptr, wfd.cFileName);
   	if ((wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
      	subdirs && strcmp (wfd.cFileName, ".") != 0 &&
         strcmp (wfd.cFileName, "..") != 0 &&
			find_file_time (pathname, filename, filetime)) {

         FindClose (h);
         return TRUE;
      }
   } while (FindNextFile (h, &wfd));
   FindClose (h);
   return FALSE;
}


